package
{
	import com.adobe.media.SphericalVideo;
	
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.NetStatusEvent;
	import flash.geom.Rectangle;
	import flash.net.NetConnection;
	import flash.net.NetStream;
	
	[SWF(frameRate="30", width="800",height="600", backgroundColor="#808080")]
	public class SphericalVideoSamplePlayer extends Sprite
	{
		private static const FILE_NAME:String = "360VideoClip.mp4";
		private static const BORDER:Number = 20;
		private static const FOV_Y:Number = 70;
		
		private var sphericalVideo:SphericalVideo;
		
		private var nc:NetConnection;
		private var ns:NetStream;
		private var totalTime:Number = 0;

		private var thumb:Shape;
		private var interactiveThumb:Sprite;
		private var rect:Rectangle = new Rectangle(0, 0, 0, BORDER);

		private var isSpherialVideo:Boolean = false;
		private var projectionType:int = SphericalVideo.PROJECTION_NONE; 
		protected var pitch:Number = 0;
		protected var yaw:Number = 0;
		protected var roll:Number = 0;
		
  		public function SphericalVideoSamplePlayer()
		{
			addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////		
		private function onAddedToStage(event:Event):void
		{
			// Scaling
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			
			// Thumb seek Bar
			thumb = new Shape();
			interactiveThumb = new Sprite();
			interactiveThumb.addChild(thumb);
			addChild(interactiveThumb);

			sphericalVideo = new SphericalVideo(stage, 0, 0, 800, 500);		
			sphericalVideo.addEventListener(SphericalVideo.AVAILABLE, onSphericalVideoAvailable);
			
			stage.addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			
			updateLayout();
		}

		//////////////////////////////////////////////////////////////////////////////////////////
		private function onSphericalVideoAvailable(e:Event):void
		{					
			nc = new NetConnection();
			nc.connect(null);
			ns = new NetStream(nc);
			ns.addEventListener(NetStatusEvent.NET_STATUS, onNetStatus);
			ns.client = this;
			ns.bufferTime = 2;
			sphericalVideo.attachNetStream(ns);

			ns.play(FILE_NAME);
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////
		private function onEnterFrame(e:Event):void
		{
			if (!ns) return;
			var play_ratio:Number = (ns.time / totalTime) * (stage.stageWidth - (BORDER * 2));
			rect.width = play_ratio;
			thumb.graphics.clear();
			thumb.graphics.beginFill(0xFFFFFF);
			thumb.graphics.drawRect(rect.x, rect.y, rect.width, rect.height);
		}

		private function updateLayout():void
		{
			interactiveThumb.x = BORDER;
			interactiveThumb.y = stage.stageHeight - (BORDER * 2);
		}
				
		//////////////////////////////////////////////////////////////////////////////////////////
		private var oldX:int = 0;
		private var oldY:int = 0;
		private var trackMouseMove:Boolean = false;
		
		protected function onMouseDown(event:MouseEvent):void
		{
			if ( event.stageY >= interactiveThumb.y - BORDER && event.stageX <= stage.stageWidth - BORDER )
			{
				var seekTime:Number = (stage.mouseX - BORDER) * ( totalTime / (stage.stageWidth - (BORDER * 2) ) );
				ns.seek( seekTime );	
			}
			else if (isSpherialVideo)
			{
				trackMouseMove = true;
				oldX = event.stageX;
				oldY = event.stageY;
			}
		}
		
		protected function onMouseMove(event:MouseEvent):void
		{
			if (!trackMouseMove) return

			var newX:Number = event.stageX;
			var newY:Number = event.stageY;
			var deltaX:Number = (newX - oldX) * FOV_Y / 360.0;
			var deltaY:Number = (newY - oldY) * FOV_Y / 360.0;
			
			oldX = newX;
			oldY = newY;			
			
			pitch += deltaY;
			if (pitch < -90) 
				pitch = -90;
			else if (pitch > 90) 
				pitch = 90;
			
			yaw += deltaX;
			if (yaw < -180) 
				yaw = 180;
			else if (yaw > 180) 
				yaw = -180;
			
			sphericalVideo.setCameraView(pitch, yaw, 0);
		}
		
		protected function onMouseUp(event:MouseEvent):void
		{
			trackMouseMove = false;
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////		
		// Netstream related event handler
		private function parseXML(info:XML):void
		{
			for each(var child:XML in info.children()) 
			{
				if (child.localName() == "Spherical")
				{
					isSpherialVideo = child.valueOf();
				}
				else if (child.localName() == "ProjectionType")
				{
					if (child.valueOf() == "equirectangular")
					{
						isSpherialVideo = true; // Assuming that video is spherical
						projectionType = SphericalVideo.PROJECTION_EQUIRECTANGLUAR;
					}
					else if (child.valueOf() == "cubemap")
					{
						isSpherialVideo = true; // Assuming that video is spherical
						projectionType = SphericalVideo.PROJECTION_CUBEMAP;
					}
				}
				else if (child.localName() == "InitialViewHeadingDegrees")
				{
					yaw = child.valueOf(); 
				}
				else if (child.localName() == "InitialViewPitchDegrees")
				{
					pitch = child.valueOf(); 
				}
				else if (child.localName() == "InitialViewRollDegrees")
				{
					roll = child.valueOf(); 
				}
				else if (child.children().length() > 0)
				{
					parseXML(child);
				}
			}
		}
		
		public function onMetaData (event:Object):void
		{
			totalTime = event.duration;

			isSpherialVideo = false;
			projectionType = SphericalVideo.PROJECTION_NONE; 

			if (event.hasOwnProperty("sphericalVideoInfo") && event.sphericalVideoInfo[0])
				parseXML(new XML(event.sphericalVideoInfo[0]));

			sphericalVideo.setProjectionType(projectionType);

			if (isSpherialVideo)
			{
				var fov:Number = FOV_Y;			
				var zNear:Number = 0.1;
				var zFar:Number = 1000;
				
				sphericalVideo.setFOV(fov, zNear, zFar);
				sphericalVideo.setCameraView(pitch, yaw, roll);
			}
		}
		public function onNetStatus(event:NetStatusEvent):void {}
		public function onCuePoint(info:Object):void {}
		public function onXMPData(info:Object):void {}
	}
}