package com.adobe.media
{
	import com.adobe.media.Projection;
	public class EquirectangularProjection extends Projection
	{
		private static const COLUMNS:Number = 40; // Pre-defined total number of column of sphere mesh covering 360 degree. we can get better image quality if increasing the number but it would affect performance.
		private static const ROWS:Number = 20;    // Pre-defined total number of row for sphere mesh covering 180 degree.
		
		public function EquirectangularProjection():void {}
		
		//////////////////////////////////////////////////////////////////////////////////////////
		// Vertices have been calculated with an assumption that video is equirectangularly projected to a frame. 
		// The generated vertices indicate corresponding points onto a sphere shaped mesh.
		// The sphere shaped mesh consists of 40 x 20 x 2 of triangles
		override protected function generateVertices():void
		{
			var deltaA:Number = Math.PI / ROWS;
			var deltaB:Number = 2.0 * Math.PI / COLUMNS;

			for (var row:int = 0; row <= ROWS; row++)
			{
				var a:Number = row * deltaA;
				var y:Number = Math.cos(a);
				var tv:Number = row / ROWS;
				
				for (var col:int = 0; col <= COLUMNS; col++)
				{
					var b:Number = col * deltaB - (0.5 * Math.PI); // Needs to pre-rotate -90 degree
					var x:Number = Math.sin(a) * Math.cos(b);
					var z:Number = Math.sin(a) * Math.sin(b);
					var tu:Number = (COLUMNS - col) / COLUMNS;
					vertices.push(x, y, z, tu, tv);
				}
			}
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////
		// based on the generated vertices, creating indices using vertices covering of the entire sphere shaped mesh.
		override protected function generateIndices():void
		{
			for (var row:uint = 1; row <= ROWS; row++)
			{
				var topIndex:uint = (COLUMNS + 1) * (row);
				var bottomIndex:uint = (COLUMNS + 1) * (row - 1);
				for (var col:uint = 0; col <= COLUMNS; col++)
				{
					var tl:uint = topIndex + col;
					var tr:uint = topIndex + ((col + 1) % (COLUMNS + 1));
					var bl:uint = bottomIndex + col;
					var br:uint = bottomIndex + ((col + 1) % (COLUMNS + 1));
					indices.push(tl, tr, bl); // Make a triangle of upper left part	of a rectangle			
					indices.push(br, bl, tr); // Make a triangle of lower right part of a rectangle	
				}
			}
		}
		

	}
}